<?php
namespace WCProductsDependencies;

/**
 * Legacy Main Class
 *
 * @class Legacy
 * @version 1.3.0
 */
class Legacy
{
    public static $lastChangedVersion = '1.17.0';

    /** Class constructor */
    public function __construct()
    {
        $this->productsUpdate();
        $this->variationsUpdate();
        $this->termsUpdate();
    }

    /** Products update */
    public function productsUpdate()
    {
        $products = get_posts([
            'numberposts' => -1,
            'post_type' => 'product'
        ]);

        foreach ($products as $product) {
            // migrate meta from v1.8.1
            $metaValue = get_post_meta($product->ID, '_products_dependencies', 1);

            if (!empty($metaValue) && is_array($metaValue) && !isset(reset($metaValue)['products'])) {
                foreach ($metaValue as &$metaValueItem) {
                    $metaValueItem = [
                        'products' => $metaValueItem,
                        'inner_relation' => 'and',
                        'outer_relation' => 'or'
                    ];
                }

                update_post_meta($product->ID, '_products_dependencies', $metaValue);
            }

            // migrate meta from v1.13.0
            $metaValue = get_post_meta($product->ID, '_minimum_quantity', 1);

            if (is_integer($metaValue)) {
                $metaValue = [
                    'count' => $metaValue,
                    'type' => 'count',
                    'terms' => []
                ];

                update_post_meta($product->ID, '_minimum_quantity', $metaValue);
            }

            $keysToReplace = [
                '_products_dependencies' => '_wcpd_products_dependencies', // @since 1.16.0
                '_terms_dependencies' => '_wcpd_terms_dependencies',
                '_quantity_dependency' => '_wcpd_quantity_dependency',
                '_incompatible_products' => '_wcpd_incompatible_products',
                '_logged_dependency' => '_wcpd_logged_dependency',
                '_visibility_by_users' => '_wcpd_visibility_by_users',
                '_enable_individual_minimum_quantity' => '_wcpd_enable_individual_minimum_quantity',
                '_minimum_quantity' => '_wcpd_minimum_quantity',
                '_enable_custom_message' => '_wcpd_enable_custom_message',
                '_custom_message' => '_wcpd_custom_message',
                '_wcpd_enable_individual_minimum_quantity' => '_wcpd_enable_individual_min_quantity', // @since 1.17.0
                '_wcpd_minimum_quantity' => '_wcpd_min_quantity'
            ];

            foreach ($keysToReplace as $old => $new) {
                $metaValue = get_post_meta($product->ID, $old, 1);

                if (!empty($metaValue)) {
                    update_post_meta($product->ID, $new, $metaValue);
                    delete_post_meta($product->ID, $old);
                }
            }
        }
    }

    /** Variations update */
    public function variationsUpdate()
    {
        $products = get_posts([
            'numberposts' => -1,
            'post_type' => 'product_variation'
        ]);

        foreach ($products as $product) {
            // migrate meta from v1.8.1
            $metaValue = get_post_meta($product->ID, '_variations_dependencies', 1);

            if (empty($metaValue) || !is_array($metaValue) || isset(reset($metaValue)['products'])) {
                continue;
            }

            foreach ($metaValue as &$metaValueItem) {
                $metaValueItem = [
                    'products' => $metaValueItem,
                    'inner_relation' => 'and',
                    'outer_relation' => 'or'
                ];
            }

            update_post_meta($product->ID, '_variations_dependencies', $metaValue);

            $keysToReplace = [
                '_variations_dependencies' => '_wcpd_variations_dependencies', // @since v1.16.0
            ];

            foreach ($keysToReplace as $old => $new) {
                $metaValue = get_post_meta($product->ID, $old, 1);

                if (!empty($metaValue)) {
                    update_post_meta($product->ID, $new, $metaValue);
                    delete_post_meta($product->ID, $old);
                }
            }
        }
    }

    /** Categories update */
    public function termsUpdate()
    {
        $terms = get_terms([
            'taxonomy' => 'product_cat',
            'hide_empty' => false,
            'hierarchical' => false
        ]);

        foreach ($terms as $term) {
            // migrate meta from v1.8.1
            $metaValue = get_term_meta($term->term_id, '_terms_dependencies', 1);

            if (!empty($metaValue) && is_array($metaValue) && !isset(reset($metaValue)['terms'])) {
                foreach ($metaValue as &$metaValueItem) {
                    $metaValueItem = [
                        'terms' => $metaValueItem,
                        'inner_relation' => 'and',
                        'outer_relation' => 'or'
                    ];
                }

                update_term_meta($term->term_id, '_terms_dependencies', $metaValue);
            }

            // migrate meta from v1.13.0
            $keys = [
                '_minimum_product_quantity',
                '_minimum_products_quantity',
                '_minimum_products_selected',
            ];

            foreach ($keys as $key) {
                $metaValue = get_term_meta($term->term_id, $key, 1);

                if (is_integer($metaValue)) {
                    $metaValue = [
                        'count' => $metaValue,
                        'type' => 'count',
                        'terms' => []
                    ];

                    update_term_meta($term->term_id, $key, $metaValue);
                }
            }

            $keysToReplace = [
                '_logged_dependency' => '_wcpd_logged_dependency', // @since 1.16.0
                '_terms_dependencies' => '_wcpd_terms_dependencies',
                '_quantity_dependency' => '_wcpd_quantity_dependency',
                '_incompatible_terms' => '_wcpd_incompatible_terms',
                '_minimum_product_quantity' => '_wcpd_minimum_product_quantity',
                '_minimum_products_quantity' => '_wcpd_minimum_products_quantity',
                '_minimum_products_selected' => '_wcpd_minimum_products_selected',
                '_prepayment_bundle_size' => '_wcpd_prepayment_bundle_size',
                '_prepayment_bundle_regularity' => '_wcpd_prepayment_bundle_regularity',
                '_enable_custom_message' => '_wcpd_enable_custom_message',
                '_custom_message' => '_wcpd_custom_message',
                '_wcpd_minimum_product_quantity' => '_wcpd_min_product_quantity', // @since 1.17.0
                '_wcpd_minimum_products_quantity' => '_wcpd_min_products_quantity',
                '_wcpd_minimum_products_selected' => '_wcpd_min_products_selected'
            ];

            foreach ($keysToReplace as $old => $new) {
                $metaValue = get_term_meta($term->term_id, $old, 1);

                if (!empty($metaValue)) {
                    update_term_meta($term->term_id, $new, $metaValue);
                    delete_term_meta($term->term_id, $old);
                }
            }
        }
    }
}
